//
//  XSPEC12  November 2003
//
//

#include <xsTypes.h>
#include <XSModel/GlobalContainer/ModelContainer.h>
#include <XSUtil/FunctionUtils/funcType.h>
#include <cmath>
#include <iomanip>

#include <Cluster.h>

extern "C"  void ascaCluster(const EnergyPointer& energyArray,
                                  const std::vector<Real>& parameterValues,
                                  GroupFluxContainer& flux,
                                  GroupFluxContainer& fluxError,
                                  MixBase** mixGenerator,
                                  const string& modelName)
{
   using namespace XSContainer;
   static const string CLUSTER("Cluster");
   // mixBase pointers are owned by ModelContainer which is a singleton.
   ModelContainer* m(ModelContainer::Instance());

   // on first call, set the pointer addressed by mixGenerator to 
   // be returned to the calling routine.
   try
   {
      if (!*mixGenerator)
      {

            if (!m->mixingComponents(CLUSTER))
            {
                      m->mixingComponents(CLUSTER,new Cluster(CLUSTER));   
            }
            *mixGenerator = m->mixingComponents(CLUSTER);
            (*mixGenerator)->initialize(parameterValues, modelName);
      }

      (*mixGenerator)->refreshTransform();
      (*mixGenerator)->perform(energyArray,parameterValues,flux,fluxError);
   }
   catch (...)
   {
      // If it throws out of here, MixComponent::perform may never
      // get the chance to set its mixGenerator pointer.  Therefore,
      // can't rely on it to initiate destruction of the faulty
      // MixBase object.  So, do it now.
      m->removeMixingComponent(CLUSTER);
      *mixGenerator = 0;
      throw;
   }
}
